Attribute VB_Name = "ModExample1"
Option Explicit

' ===============================================
' Example 1: The hard way to use the MenuHelp function.
' ===============================================

' This example (as opposed to Example 2 which also uses the
' MenuHelp function) is probably the most difficult to understand
' But apparently we're going to be using the Menuhelp function
' exactly how it was designed to be used. It's a fairly intricate
' function, and for this reason, may be why it was never
' documented completely.

' If you happen to find yourself getting lost following this example,
' jump to Example 2 where the concepts much simpler but has
' a bit more working code.

' ===============================================

' This particular implementation *assumes* VB's sequential
' assignment of menu command IDs (see the included file
' VBCmdIDs.txt file for more info) and also utilizes an array
' containing submenu information which is passed as MenuHelp's
' last param, lpwIDs. The result is quite a bit less working code
' than is used in Example2 but requires a slightly more involved
' resource file. See the top of the MenuStrs.rc file.

' Note: VB normally assigns menu command IDs in the order of
' menu item appearance when the form containing the menu items
' is created. How VB assigns menu command IDs for dynamically
' created menu items after form creation may vary, i.e. menu arrays
' & MDI window lists. This assumption of ID assignment may not be
' appropriate for all applications.

' ================================================

' As one alternative to handling submenu string resource IDs, the
' structure below can be used and passed to MenuHelp's lpwIDs
' parameter.

Type MENUHELPID   ' mhid
    hMenu As Long       ' submenu handle
    wStringID As Long   ' resource string ID of 1st menu item in hMenu
End Type

' In order for this structure to be effective, it should be placed in
' an array and contain the first menu item of each submenu, that
' contains a submenu. The array below will be used and filled in
' the InitExample1() procedure.

Dim m_MHID() As MENUHELPID

' When a submenu is selected, because MenuHelp itself will not
' search the array, we will search the array ourselves finding the
' element in the array that matches the corresponding values
' returned by the WM_MENUSELECT message. This specific
' structure element will then be passed to MenuHelp's lpwIDs
' parameter.

' To have MenuHelp correctly evaluate resource ID value from
' the structure, it's hMainMenu parameter must be set to the
' value of the structure's hMenu member.

' ================================================

' The constants below mirror the values defined for Example1 in
' MenuStrs.rc. We had to define these values so MenuHelp
' references the correct submenu resource IDs when it sees
' the structure passed to it's lpwIDs parameter.

Public Const IDS_MENU1 = 100    ' "menu1", submenu
Public Const IDS_MENU2 = 101    ' "meuu2", submenu
Public Const IDS_DUMMY = 150   ' takes the place of "menu1a", is a menu command
Public Const IDS_MENU1B = 151  ' "menu1b", submenu
Public Const IDS_MENU2A = 250  ' "menu2a", submenu

' VB's positional menu command ID assignment (as decribed in
' VBCmdID.txt) could not be used for the submenus. Here's an
' example why:

' Let's say the submenu "menu2" is clicked (hilited). In DoExample1(),
' we will search the array (filled in InitExample1()) for it's parent menu
' handle, the main menu. We will pass the address of this member to
' MenuHelp's lpwIDs param and specify the parent menu handle as
' MenuHelp's hMainMenu param so that it knows it's receiving two
' long values. MenuHelp will then add the zero-based position of
' "menu2", the value 1, to the corresponding wStringID value in the
' structure, IDS_MENU1 (the ID for "menu1") and use the total 101,
' or the value IDS_MENU2, as the string resource ID to load.

' If we used VB's positional menu command ID assignment for the
' "menu2" submenu, the resource ID for "menu1" would be the value 1.
' Since the ID value is added to the position 1, resource ID 2 would be
' loaded, displaying "menu1a".

' Likewise, we really didn't need to define the IDS_DUMMY or
' IDS_MENU1B values. But if there was another submenu under
' "menu1b", it's fairly clear to see that this would be needed.
 

Public Sub InitExample1()

  Dim hMenu As Long
  Dim hMenu1 As Long
  Dim hMenu2 As Long
  
  ' Each end of the array (elements 0 & 4) must be padded
  ' with empty values for two reasons. First, when the
  ' selected menu item is a menu command, we will not
  ' search the array and it must equal 0 so that no value is
  ' added to the wParam low word menu command ID value.
  ' Secondly, if by chance the array is searched and the
  ' parent menu handle being sought is not found, we won't
  ' produce an error.
  ReDim m_MHID(4)
  
  ' Again, add the parent menu handle & resource ID of the 1st
  ' menu item, of each submenu, that contains a submenu.
  
  ' Main menu handle, contains submenus "menu1" and "menu2"
  hMenu = GetMenu(Form1.hwnd)
  ' "menu1" handle, contains submenu "menu1b"
  hMenu1 = GetSubMenu(hMenu, 0)
  ' "menu2" handle, contains submenu "menu2a"
  hMenu2 = GetSubMenu(hMenu, 1)
  
  ' Padding...
'  m_MHID(0).hMenu = 0
'  m_MHID(0).wStringID = 0
  
  m_MHID(1).hMenu = hMenu   ' main menu
  m_MHID(1).wStringID = IDS_MENU1
  
  ' We had to create a dummy placeholder item for "menu1a"
  ' with it's own resource string ID. MenuHelp uses this ID
  ' as a reference, *only* for displaying the resource strings
  ' for submenu items within "menu1".
  m_MHID(2).hMenu = hMenu1   ' "menu1"
  m_MHID(2).wStringID = IDS_DUMMY
  m_MHID(3).hMenu = hMenu2   ' "menu2"
  m_MHID(3).wStringID = IDS_MENU2A
  
  ' More padding...
'  m_MHID(4).hMenu = 0
'  m_MHID(4).wStringID = 0

End Sub

' The message loop...

Public Sub DoExample1(wParam As Long, _
                                    lParam As Long)
  Dim dwFlags As Long
  Dim wIdx As Integer
  Dim hMainMenu As Long

  Form1.Picture1.Cls
  
  ' The high order word of the wParam contains the menu
  ' item's flags
  dwFlags = ((wParam And &HFFFF0000) \ &HFFFF&)
  
  ' If still in menu mode...
  If dwFlags <> &HFFFF& And lParam Then
    
    ' Since MenuHelp does *not* search the array pointed to
    ' by it's lpwIDs parameter, we have to search it ourselves.
    
    ' For non-system submenu only, determine the array
    ' index (wIdx) for the 1st menu item of the selected submenu
    ' Also, since we're sending as structure to the lpwIDs param,
    ' assign the parent menu handle to hMainMenu so MenuHelp
    ' recognizes the structure and uses the value stored in it's
    ' wStringID member.
    If (dwFlags And MF_POPUP) And _
                (dwFlags And MF_SYSMENU) = 0 Then
      For wIdx = 1 To 3
        If lParam = m_MHID(wIdx).hMenu Then
          hMainMenu = lParam: Exit For
        End If
      Next
      
      ' If for some reason the parent menu handle isn't found
      ' in the array (which in our case probably won't happen),
      ' we'll hit the padding when the For-Next loop increments
      ' wIdx to 4, and we won't get a "subscript out of range"
      ' error below.
    End If
  
    ' Visual aid...
    Form1.Picture1.Print "wParam (low): " & (wParam And &HFFFF&)
    Form1.Picture1.Print "m_MHID(" & wIdx & ").wStringID: " & m_MHID(wIdx).wStringID
    Form1.Picture1.Print "hmenuParent: " & "&H" & Hex(lParam)
  End If   ' >> End  menu mode
  
  ' What MenuHelp does with it's parameters...
  
  ' Non-system submenu: Adds the submenu's position
  ' (wParam low word) to the submenu resource ID pointed to by
  ' the wStringID member of the structure found above in m_MHID().
  ' (wIdx & hMainMenu set above)
  
  ' Non-system menu command: Adds the menu command's
  ' ID (wParam low word) to the submenu's resource ID pointed to by
  ' the wStringID member of the structure found above in m_MHID().
  ' (wIdx = 0, hMainMenu = 0)
  
  ' System submenu and submenu system commands: Displays
  ' the system submenu help string from Comctl32.dll's stringtable
  ' resource whose resource string ID equal the values contained
  ' in wParam and lParam.
  ' (wParam = 0, wIdx = 0, hMainMenu = 0)
  
  MenuHelp WM_MENUSELECT, _
                  wParam, _
                  lParam, _
                  hMainMenu, _
                  App.hInstance, _
                  g_hStatBar, _
                  m_MHID(wIdx)
End Sub

