Attribute VB_Name = "ModExample2"
Option Explicit

' =============================================
' Example 2: The easy way to use the MenuHelp function.
' =============================================

' In this example, we're going to do things a bit differently than
' was done in Example 1. Most importantly we will not directly
' use VB's positional menu command ID scheme in establishing
' menu item resource IDs, since it may not apply in all applications.
' Instead we will use a similar postional menu item arrangement
' that will allow us to define our own specific resource IDs for all
' menu items (not just submenus). See the bottom section of the
' MenuStrs.rc file.

' Also similar to Example 1, the procedure EnumMenuItems() is
' used to sequentially load not just some, but all, menu items
' into a structure that will be elements of an array. On receipt of
' WM_MENUSELECT messages, the procedure GetResStrID()
' is called to search the array for the selected menu item and
' returns the menu item's zero-based position in the array. Any
' necessary calculations can be performed with this value in
' order to align it with our corresponding string resource ID
' structure.

' In this and in the following 3 examples, each menu item's
' position in the array is a value of 10 less than the menu item's
' corresponding resource string ID value. After adding 10 to the
' array index, the value will then be passed directly to the
' MenuHelp function's wParam parameter.

' =============================================

' The structure used to store each menu item's pertinent
' attributes which is searched by GetResStrID(). Notice that
' it contains the same information returned from the
' WM_MENUSELECT message's wParam and lParam values.

Type MENUITEMATTRIBS   ' MIA
    
    ' If a submenu, contains it's pos, item ID for menu commands.
    ' Same as WM_MENUSELECT's wParam low word value.
    wParamLow As Long
    
    ' Flag set if menu item is a submenu, providing the correct
    ' interpretation of wParamLow. Same as WM_MENUSELECT's
    ' wParam high word value equaling MF_POPUP.
    bIsSubMenu As Boolean
    
    ' Menu item's parent menu handle or lParam value.
    hmenuParent As Long

End Type

' EnumMenuItems() dimensions & loads the array.
Dim m_aMIA() As MENUITEMATTRIBS
 

Public Sub InitExample2()
  EnumMenuItems GetMenu(Form1.hwnd)
End Sub

' Recursively loads the app's entire menu structure into the
' m_aMIA() array of structures, in the same order as each
' menu item's position in the app's menu structure.

' In this example, the index of each menu item's position
' in the array equals the item's menu help string resource
' ID + 10. The value is then retrieved by GetResStrID() below.

' The resource strings for menu items dynamically created
' after the window itself is first created could be pre-defined.
' After the array is initially filled below, new items could then
' be specifically added to the array of structures at the
' element position that matches their corresponding resource
' string ID.

Public Sub EnumMenuItems(hmenuParent As Long)

  Dim wItems As Integer
  Dim wPos As Integer
  Dim hSubMenu As Long
  Static stwIndex As Integer
  
  wItems = GetMenuItemCount(hmenuParent)
  For wPos = 0 To wItems - 1
        
    ' Zero based array, starting at 1
    stwIndex = stwIndex + 1
    ReDim Preserve m_aMIA(stwIndex)
    
    ' Store the item's parent menu handle
    m_aMIA(stwIndex).hmenuParent = hmenuParent
    
    hSubMenu = GetSubMenu(hmenuParent, wPos)
  
    If hSubMenu Then
      ' Is a submenu, store it's pos within it's parent menu
      m_aMIA(stwIndex).wParamLow = wPos
      m_aMIA(stwIndex).bIsSubMenu = True
      
      ' Recursively calls itself, enumerating the current submenu
      EnumMenuItems hSubMenu
    
    Else
      ' Is a menu command, store it's command ID
      m_aMIA(stwIndex).wParamLow = GetMenuItemID(hmenuParent, wPos)
      ' m_aMIA(stwIndex).bIsSubMenu = False
    
    End If
  Next

End Sub

' The message procedure...

' Since we have an array containing all of the pertinent menu
' item information, we will be passing the exact resource
' string ID values to MenuHelp's wParam parameter. This
' allows the hMainMenu and lpwIDs parameters to be ignored.

' But in order to display the system submenu help strings, both
' the lParam value must be passed directly to MenuHelp's
' lParam parameter and lpwIDs must point to a Long data
' type which must equal 0. So the empty dwZeroVal variable
' will be passed to lpwIDs for every menu item.

Public Sub DoExample2(wParam As Long, _
                                    lParam As Long)
  Dim dwFlags As Long
  Dim dwZeroVal As Long

  Form1.Picture1.Cls
  
  ' The high order word of the wParam contains the selected
  ' menu item's flags
  dwFlags = ((wParam And &HFFFF0000) \ &HFFFF&)
  
  ' If still in menu mode...
  If dwFlags <> &HFFFF& And lParam Then
    
    If (dwFlags And MF_SYSMENU) = 0 Then
      ' A non-system submenu or menu command, pass the
      ' necessary values to GetResStrID(). It will set wParam
      ' to the menu items exact resource string ID value.
      ' wParam low word (IN) :
      '   If a submenu, is zero-based pos within it's parent menu.
      '   If a menu command, is the item's command ID.
         wParam = GetResStrID(wParam And &HFFFF&, _
                                            dwFlags And MF_POPUP, _
                                            lParam)
    End If
  
    Form1.Picture1.Print "wParam (low): " & (wParam And &HFFFF&)
    Form1.Picture1.Print "hmenuParent: " & "&H" & Hex(lParam)
  End If   ' >> End menu mode >>
  
  ' As oppsed to Example 1, things are much simpler here We
  ' took care of the wParam value above for non-system menu
  ' items. MenuHelp will again use the wParam and lParam values
  ' returned from the WM_MENUSELECT message to display
  ' the system submenu help strings.
  MenuHelp WM_MENUSELECT, _
                  wParam, _
                  lParam, _
                  0, _
                  App.hInstance, _
                  g_hStatBar, _
                  dwZeroVal
End Sub

' Walks through the m_aMIA() array of structures &
' returns the one-based index of the menu item's entry
' in the array which matches the passed params.

' See the definition of the MENUITEMATTRIBS
' structure in the declaration section of this module.

Private Function GetResStrID(wParamLow As Long, _
                                            bIsSubMenu As Boolean, _
                                            hmenuParent As Long) As Integer
  Dim wIdx As Integer
  
  For wIdx = 1 To UBound(m_aMIA)
    With m_aMIA(wIdx)
      If .wParamLow = wParamLow And _
          .bIsSubMenu = bIsSubMenu And _
             .hmenuParent = hmenuParent Then
        
        ' 10 is the value added to each string resource ID
        GetResStrID = wIdx + 10
        Exit Function
      
      End If
    End With
  Next

End Function
