Attribute VB_Name = "ModExample3"
Option Explicit

' =============================================
' Example 3: Why even bother with the MenuHelp function?
' =============================================

' Demonstrates how to place menu help text in a status bar
' without the use of the MenuHelp function. Messages are
' explicitly sent to the status bar, simulating MenuHelp's
' functionality.

' =============================================

' This example introduces some interesting concepts...

' In Example2, we loaded all of the app's main menu items into
' an array and used the value of the item's index in the array to
' determine it's corresponding resource string ID. Instead, in
' this example we'll use a function similar to the one we used in
' Example2 to load the array, called GetMenuItemPos(). It
' will dynamically determine the selected menu item's relative
' position in the app's menu structure by checking the
' (MF_HILITE) menu state. As a result, we won't either need or
' be using both the wParam and lParam values returned by the
' WM_MENUSELECT message.

' Note: Because GetMenuItemPos() will rely on the selected
' menu item's position with reagrd to the current menu structure,
' as in Example 1, no accurate means of obtaining dynamically
' created items can be provided. This function is introduced
' only to prepare for Example 4. The EnumMenuItems()
' function used in conjuction with the MENUITEMATTRIBS
' array of structures still can and should be used instead.

' Again, in this example item's position in the array is a value of
' 10 less than the menu item's corresponding resource string ID
' value. But after adding a value of 10 to the array index, the
' value will then be passed directly to VB's LoadString function.

' If the selected menu item is found to be part of the system
' submenu, the system menu help strings will now be loaded
' directly from Comctl32.dll stringtable resource by using our
' GetSysMenuStr() function.

' Finally, because the MenuHelp won't be used, to simulate it's
' functionality, we will both toggle the status bar's simple mode
' accordingly with the SB_SIMPLE message, and draw the help
' strings on it with the SB_SETTEXT message using the
' SBT_NOBORDERS drawing operation flag.

' =============================================

' Form1's main menu handle
Dim m_hMainMenu As Long
 

Public Function InitExample3()
  ' We only need to initialize one local variable
  m_hMainMenu = GetMenu(Form1.hwnd)
End Function

' The message procedure......

' As a result of not employing the services of the MenuHelp
' function, we will to do all of it's work.

Public Sub DoExample3(wParam As Long, _
                                    lParam As Long)
  Dim dwFlags As Long
  Dim sText As String

  ' The high order word of the wParam contains the menu
  ' item's flags
  dwFlags = ((wParam And &HFFFF0000) \ &HFFFF&)
  
  If dwFlags <> &HFFFF& And lParam Then
    ' In menu mode...
    
    If (dwFlags And MF_SYSMENU) = 0 Then
      ' Not the system menu or one of it's menu commands,
      ' retrieve the string from our resource file (add 10).
      sText = LoadResString(GetMenuItemPos(m_hMainMenu, False) + 10)
    
    Else
      ' A system submenu or system menu command,
      If dwFlags And MF_POPUP Then
        ' The system menu itself, retrieve it's string
        ' using Comctl32.dll's specific resource string ID.
        sText = GetSysMenuStr(36848)
      Else
        ' A system menu command, subtract the magic # from
        ' the command's ID & retrieve the string from Comctl32.dll
        sText = GetSysMenuStr((wParam And &HFFFF&) - &H7000&)
      End If
    
    End If
    
    ' Make the status bar simple & set the text
    SendMessage g_hStatBar, SB_SIMPLE, ByVal 1, 0
    SendMessage g_hStatBar, SB_SETTEXT, _
                          ByVal 255 Or SBT_NOBORDERS, ByVal sText
  
  Else   ' (dwFlags = &HFFFF& And lparam = 0)
    ' Not in menu mode anymore, restore
    ' the status bar to it's original state
    SendMessage g_hStatBar, SB_SIMPLE, ByVal 0, 0
    
  End If   ' >> End menu mode >>
      
    
End Sub

' This is interesting...

' The system menu help strings are found in Comctl32.dll's
' stringtable resource. The system menu command IDs are
' shared system wide and are all a value of &H7000 greater
' than Comctl32.dll's corresponding resource string IDs.
' The system submenu's help string is extracted with it's
' explicit resource string ID in the msg hook proc above.

' dwStringID: The resource string ID of the string to retrieve.
' Rtns the resource string if successfull, an empty string on error.

Public Function GetSysMenuStr(dwStringID As Long) As String
  Dim hMod As Long
  Dim sMsgBuf As String * 260
  Dim dwLen As Long
  
  hMod = GetModuleHandle("comctl32")
  dwLen = LoadString(hMod, dwStringID, sMsgBuf, 260)
  If dwLen Then GetSysMenuStr = Left$(sMsgBuf, dwLen)
  
End Function

' This function rtns either the one-based menu position of the
' currently selected menu item or a count of the menu items in
' the specified menu.

' hMenu:   The current submenu to search.
' bRtnVal: Flag checked at the end of the proc which determines
'               whether the count of unhighlighted menu items should
'               be rtnd.

' Return values:
' - If bRtnVal = False & a menu item is hilited, rtns it's one-based
'   position within the submenu specified by hMenu.
' - If bRtnVal = False & no menu item in the submenu is hilited, rtns 0.
'  -If bRtnVal = True, rtns the number of menu items in the submenu
'   specified by hMenu, regardless of any hilited items.

Public Function GetMenuItemPos(hMenu As Long, _
                                                  bRtnVal As Boolean) As Long
  Dim wItems As Integer
  Dim wPos As Integer
  Dim hSubMenu As Long
  Dim wRtn As Integer
  
  wItems = GetMenuItemCount(hMenu) - 1
  For wPos = 0 To wItems
    
    wRtn = wRtn + 1  ' add every item
    hSubMenu = GetSubMenu(hMenu, wPos)
    
    If (GetMenuState(hMenu, wPos, MF_BYPOSITION) And MF_HILITE) = 0 Then
      ' Item isn't hilited, if a submenu, count it's items.
      If hSubMenu Then wRtn = wRtn + GetMenuItemPos(hSubMenu, True)
    
    Else
      ' Item is hilited, we're close...!
      If hSubMenu Then
        ' Is a submenu & is hilited, rtn the pos (if any) of it's hilited
        ' items + any items that were counted above & exit below.
        wRtn = wRtn + GetMenuItemPos(hSubMenu, False)
      End If
      ' If is a menu command & is hilited, just rtn it's
      ' pos + any items that were counted above & exit.
      GetMenuItemPos = wRtn
      Exit Function
    
    End If
  Next

  ' If not searching a hilited submenu, rtn the total items counted
  ' (if we don't want any menu cmds to be counted that are below
  ' a hilited submenu).
  If bRtnVal Then GetMenuItemPos = wRtn

End Function
