Attribute VB_Name = "ModExample4"
Option Explicit

' =============================================
' Example 4: Since we don't need the MenuHelp function, why
'                   bother with subclassing...?
' =============================================

' Demonstrates how to place menu help text in a status bar
' without the use of the MenuHelp function OR a subclassing
' routine to catch WM_MENUSELECT messages.

' It also employs the use of a recursive function to determine
' which particular menu item's text to display. As in Example3,
' messages are also explicitly sent to the status bar, simulating
' MenuHelp's functionality.

' =============================================
' (Long oration...)

' Since in Example3 we eliminated the need of the MenuHelp
' function and were able to determine if, and which, menu item
' is currently selected by using the GetMenuItemPos() function,
' this example attempts to replace the need for a subclassing
' procedure entirely and dynamically determine if and when any
' application menu item is selected.

' To determine whether the application is in menu mode or not,
' the combination of a Timer that fires 10 times per second and
' an abbreviated version of the GetMenuItemPos() menu highlight-
' checking routine is used. This routine, the IsMainMenuActive()
' function, checks the highlighted state (MF_HILITE) of only the
' top-level menu items in the main menu.

' Since in Example3 it was shown that the selected state of
' any of the main menu's items could be determined, the same
' could be assumed for the system submenu's menu commands.
' But what about the system submenu itself? What if the system
' submenu is popped up and none of it's commands are currently
' selected, as is when it is first clicked?

' It's MF_HILITE state can be checked with the GetMenuState()
' API function of course. But GetMenuState() requires a menu
' item's parent menu handle and the item's position (or command
' ID) as parameters. Is the system submenu a child of some other
' menu? Well, actually it is...

' The lParam value returned by the WM_MENUSELECT message
' proves this when the system submenu is first shown. And this
' parent menu only contains one item, the system submenu,
' which occupies the first position, 0.

' But how in the world do we retrieve this parent menu handle of
' the system submenu without using as subclassing routine that
' can catch the WM_MENUSELECT window message? The API
' functions that return menu handles, GetMenu(), GetSubMenu()
' and GetSystemMenu() all return handles of menus that are
' children of some parent. There is no function the works in reverse,
' returning the parent menu handle of a specified child menu item.

' This is where shear ugliness comes in to play, with the
' GetSysMenuParent() function below. The function begins at 0,
' incrementally advances one whole number at a time, and checks
' to see whether the current value is a valid menu handle with the
' IsMenu() API function. If the value is a valid menu handle, the
' function then checks to see if the current menu contains only
' one menu item at it's first position, our system submenu. If
' and when the handle of the system submenu's parent is finally
' obtained, it appears that we've successfully completed our
' task at hand...

' One last note: This example was created solely for the purpose
' of demonstration and should probably not be used as working
' code in any distributable application. There's no telling if the
' methods employed are fully reliable or will be compatible with
' future versions of Windows.

' =============================================

' A liberal amount of module level variables
' are used in this example.

' A local copy of Form1's timer
Dim objtmrMenuMode As Timer

Dim m_hMainMenu As Long
Dim m_hSysMenu As Long
Dim m_hSysMenuParent As Long
 

' Initialize all of the local variables now
' with the hope of improving performance later...

Public Function InitExample4() As Long
  
  Set objtmrMenuMode = Form1.tmrExample4
  
  m_hMainMenu = GetMenu(Form1.hwnd)
  m_hSysMenu = GetSystemMenu(Form1.hwnd, False)
  m_hSysMenuParent = GetSysMenuParent()
  
  ' Retruns True when GetSysMenuParent()
  ' hopefully find's what it's looking for, see below...
  InitExample4 = m_hSysMenuParent

End Function

' Returns the menu handle of the system menu's parent.

' This is as ugly as it gets... An app's system submenu is a child
' of some menu, but I have no idea how to obtain it other than the
' shameful routine below. Hopefully there won't have been many
' menu handles assigned & we can get this over with in a hurry :)

' BTW the demo is already loaded and shown in Form1.Form_Load
' and is fully functional when this function is called. This example
' becomes available for use only when, and IF, this function
' completes it's task. The DoEvents statement in the loop allows
' the demo to proceed, unencumbered by what's going on here
' (with the exception of the processor usage being a bit pegged...).

Public Function GetSysMenuParent() As Long
  Dim dwMenu As Long
  Dim hMenu As Long
  Dim hSysMenu As Long
  
  ' One by one, from 0 to (2 ^ 31) - 1, first test to see if the
  ' value is a valid menu handle, and if it is, then see if
  ' the handle of it's 1st menu item is our system menu...
  For dwMenu = 0 To &H7FFFFFFF
    DoEvents
    If IsMenu(dwMenu) Then
      If GetSubMenu(dwMenu, 0) = m_hSysMenu Then
        ' Once here, this example is on it's way...
        GetSysMenuParent = dwMenu: Exit Function
      End If
    End If
  Next

' We won't bother checking the 2nd half of the HMENU
' values (0x80000000 --> 0xFFFFFFFF) and call this attempt
' a complete failure if the handle isn't found above... Hasn't
' happened yet though...(!!??)

End Function

' Form1's timer is essentially the "virtual" message loop.
' This is it's code...

' The variables bSysMenuActive, sText, wItemPos and
' dwSysCmdID are static for performance considerations. The
' values of the other two variables, bMenuMode & sTextBuf
' must be maintained throughout the menu mode period.

' The sTextBuf variable is used to prevent the status bar's
' text from being set & redrawn at every timer interval,
' preventing lots of flicker.

' Most of the code below is executed at every timer interval,
' and is a compromise in order to simplify the code. A Long
' variable could be used to store the currently displayed
' menu item's respective value...

' The timer's intervals were also chosen as a compromise
' & can obviously be changed to suit personal preferance.

' Uses GetMenuItemPos() & GetSysMenuStr() in Example3.bas

Public Sub DoExample4()   ' Interval initially set to 100
  
  Static bSysMenuActive As Boolean
  Static bMenuMode As Boolean
  Static sText As String
  Static wItemPos As Integer
  Static dwSysCmdID As Long
  Static sTextBuf  As String
  
  ' The system submenu's parent only contains one item,
  ' the system submenu itself. So it's a faster to check it
  ' rather than the main menu, which could contain many
  ' more top-level menu items.
  ' This flag is also used again in a test below.
  bSysMenuActive = (GetMenuState(m_hSysMenuParent, _
                                                     0, MF_BYPOSITION) And MF_HILITE)
  
  If IsMainMenuActive() Or bSysMenuActive Then
    ' We're now in menu mode...
    
    If Not bMenuMode Then   ' (oh well...)
      ' The flag isn't set yet, initialize everything...
      bMenuMode = True
      ' Make the status bar display a little snappier...
      objtmrMenuMode.Interval = 10   ' 100 hits/sec
      ' Make the status bar simple (1 part).
      SendMessage g_hStatBar, SB_SIMPLE, ByVal 1, 0
    End If
    
    ' This block of code is identical to that which was used in the
    ' Example3 message procedure, but is flipped over so the flag
    ' can be tested without the use of VB's "Not" operator.
    If bSysMenuActive Then
      ' The system submenu or a system menu command
      ' is active, get the currently selected item.
      wItemPos = GetMenuItemPos(m_hSysMenu, False)
      If wItemPos Then
        ' A system menu command, get it's menu command ID.
        dwSysCmdID = GetMenuItemID(m_hSysMenu, wItemPos - 1)
        ' Subtract the magic # from the command's ID & retrieve
        ' it's string from Comctl32.dll.
        sText = GetSysMenuStr(dwSysCmdID - &H7000&)
      Else
        ' GetMenuItemPos rtnd 0, no item is selected, retrieve
        ' the system submenu's string using Comctl32.dll's
        ' specific resource string ID.
        sText = GetSysMenuStr(36848)
      End If
          
    Else
      ' The main menu, find the highlighted item
      ' & retrieve the string from our resource file.
      sText = LoadResString(GetMenuItemPos(m_hMainMenu, True) + 10)
    
    End If
    
    ' Set the menu item's help text if it's different
    ' than the currently displayed text, we *don't*
    ' need to redraw the status bar 100 times/sec...
    If sText <> sTextBuf Then
      sTextBuf = sText
      SendMessage g_hStatBar, SB_SETTEXT, _
                            ByVal 255 Or SBT_NOBORDERS, ByVal sText
    End If
    
  ElseIf bMenuMode Then
    ' Menu mode has terminated, restore everything.
    bMenuMode = False
    objtmrMenuMode.Interval = 100
    sTextBuf = ""
    
    ' Restore the status bar to it's original state.
    SendMessage g_hStatBar, SB_SIMPLE, ByVal 0, 0
    
  End If   ' >> End menu mode >>
      
End Sub

' Determines if any top-level submenu of the main menu
' is currently highlighted.

' Is an abreviated version of GetMenuItemPos() (Example3)
' and get's called 100 times/sec when in menu mode...

' hMenu:   The current submenu to search.
' Rtns True if an item is highlighted, False if not.

Private Function IsMainMenuActive() As Boolean
  Static wItems As Integer
  Static wPos As Integer
  
  ' wItems could be set at init and stored in a module
  ' level variable, but may cause problems if this app
  ' were a MDI client & a MDI child was maximized...
  wItems = GetMenuItemCount(m_hMainMenu)
  For wPos = 0 To wItems - 1
    If (GetMenuState(m_hMainMenu, _
                              wPos, MF_BYPOSITION) And MF_HILITE) Then
      IsMainMenuActive = True: Exit Function
    End If
  Next
End Function

'      ....Wasn't that fun...?!
