Attribute VB_Name = "ModMenuHelp"
Option Explicit

' Brought to you by:
'   Brad Martinez
'   btmtz@msn.com
'   btmtz@aol.com
'   http://members.aol.com/btmtz/vb

'======================================================
' The WM_MENUSELECT window message. The one and only message
' discussed in this demo that the MenuHelp function processes:

' The WM_MENUSELECT message is sent to a menu's owner window when
' the user selects a menu item.

' wParam low word:
' > If the selected item is a submenu, the low word of WM_MENUSELECT's
'    wParam value is the submenu's zero based position (MF_POSITION)
'    within it's parent menu.
' > If the selected item is a menu command, the low word is the menu
'    command's menu ID.

' wParam high word:
' The high word of wParam always indicates the state of the selected menu
' item.

' lParam:
' WM_MENUSELECT's lParam always indicates the handle of the menu in
' which the selected item belongs (it's parent menu), whether it's a submenu
' or a menu command.

' If the fuFlags parameter contains 0xFFFF and the lParam parameter contains
' NULL, Windows has closed the menu.

Public Const WM_MENUSELECT = &H11F

'=======================================================
' Menu state flags returned by the WM_MENUSELECT message used in
' conjunction with the MenuHelp function (wParam high word):

Public Const MF_POPUP = &H10&
Public Const MF_SYSMENU = &H2000&

'=======================================================
' The MenuHelp function:

' (Note: What follows is an extensive elaboration of the information found in
'           the book "Programming Windows 95" by Charles Petzold and Paul
'           Yao, published by Microsoft Press, 1996. Every effort has been made
'           in an attempt to provide the most clear and accurate information
'           possible. If for any reason this information is not found to be clear
'           or accurate, the author would appreciate notification at either of the
'           e-mail addresses at the top of this code module. Thank you)

' The MenuHelp function processes the WM_MENUSELECT message and
' displays help text for the currently selected menu in the specified status
' window. It's SDK documentation indicates that handling is also provided for
' the WM_COMMAND message, but it was either never fully implemented or
' well beyond the scope of this demo.

' The MenuHelp function is designed to be used in a message procedure.

' MenuHelp also automatically displays help strings for an application's system
' submenus and system submenu commands.

Declare Sub MenuHelp Lib "comctl32" (ByVal uMsg As Long, _
                                                         ByVal wParam As Long, _
                                                         ByVal lParam As Long, _
                                                         ByVal hMainMenu As Long, _
                                                         ByVal hInst As Long, _
                                                         ByVal hwndStatus As Long, _
                                                         lpwIDs As Any)

' Because the MenuHelp function may have been initially designed to handle
' both the WM_MENUSELECT and WM_COMMAND messages, it can become
' terribly complicated if all of it's parameters are taken into consideration. To
' simplify MenuHelp as much as possible, it's functionality for the
' WM_COMMAND message will be ignored.

' Parameter     Description
' uMsg            WM_MENUSELECT message.

' wParam        Added with lpwIDs to calculate the string resource ID. If the
'                     WM_MENUSELECT wParam value is used, only it's low word
'                     value is evaluated. Can be 0.

' lParam         The parent menu handle of the selected menu item. Used only
'                     for retrieving submenu help strings. Is ignored if the selected
'                     menu item is a menu command. Can be 0.

' hMainMenu   Indicates that the lpwIDs parameter points to an array or
'                     structure of submenu handle/string resource ID value pairs.
'                     This parameter must be set to the submenu handle value in
'                     order for the function to evaluate the string resource ID value
'                     correctly. Otherwise this parameter is ignored and should
'                     be set to 0. See Remarks below.

' hInst             Handle to the module that contains the string resources.

' hwndStatus   Handle to the status window.

' lpwIDs:          Pointer to a value which is added with wParam to calculate the
'                     string resource ID. May point to an array or structure of submenu
'                     handle/string resource ID value pairs as descibed above. Must
'                     point to a Long data type (4 bytes), set to a value of 0, in order
'                     for the system submenu's help string to be displayed. Otherwise
'                     this parameter can be Null. See Remarks below.

' This function does not return a value.

' ----------------------------------------------------
' Description:

' When MenuHelp receives the first WM_MENUSELECT message at menu
' mode initialization, it places it's associated status bar in "simple mode" by
' sending it a SB_SIMPLE message (wParam = True).

' While receiving the initial and subsequent WM_MENUSELECT messages,
' MenuHelp uses the WM_MENUSELECT parameters passed to it to reference
' resource strings in the specified module. It displays the strings by sending
' SB_SETTEXT messages to the status bar, drawing the text on the simple
' mode's part (255), and using the SB_NOBORDERS drawing operation.

' When MenuHelp receives a final WM_MENUSELECT message indicating
' menu mode termination (the low word in WM_MENUSELECT's wParam equals
' 0xFFFF), MenuHelp again sends the SB_SIMPLE message (wParam = False)
' to the status bar, restoring it to it's non-simple state.

' ----------------------------------------------------
' Remarks:

' Concerning the lpwIDs parameter, the Win32 documentation states:

'     "Pointer to an array that contains pairs of string resource identifiers and menu
'      handles. The function searches the array for the handle to the selected menu
'      and, if found, uses the corresponding resource identifier to load the appropriate
'      help string."

' This does not appear to be accurate. MenuHelp uses the value pointed to by
' the lpwIDs parameter to "calculate" a string ID. It does not "search" it's data.

' The function actually *adds* the value pointed to by lpwIDs to the low word of the
' wParam value (a submenu's position or a menu command's ID) . lpwIDs can point
' to a *single value*.

' To clarify this, the following shows three examples of parameter values in which
' MenuHelp will evaluate to a resource string ID value of 10:

'   wParam   +   lpwIDs  = resource ID
'     0                  10
'     5                    5
'    10                   0

' The lpwIDs parameter can also point to an element in an array (or a structure)
' that contains submenu handle/string resource ID value pairs. In this case, the
' hMainMenu parameter must be set to the submenu handle value so that the
' menu handle value is ignored, and the corresponding resource ID value is added
' to the low word of the wParam value as described above.

' ----------------------------------------------------
' How it can be used:

' Since menu commands have their own IDs, it's easiest to set lpwIDs to 0 and pass
' WM_MENUSELECT's wParam value directly to MenuHelp's wParam parameter.

' The same applies to system submenu menu commands with regard to the wParam
' value. But WM_MENUSELECT's lParam value (the system submenu) must also be
' directly passed to MenuHelp's lParam parameter.

' As well, in order for MenuHelp to display the help string of the system submenu itself,
' hMainMenu cannot be set to the lParam value (the system submenu's parent menu)
' if the lpwIDs parameter points to an array or structure. Otherwise lpwIDs must only
' point to a Long data type which should be set to 0 or Null.

' Non-system submenus pose a bit of a problem. Since the low word of the wParam
' value returned by the WM_MENUSELECT message contains the submenu's position
' with respect to it's parent menu, there's no direct way of associating the selected
' submenu with a particular string resource ID value without programmatically
' determining the value. This is where the hMainMenu parameter becomes significant
' and will be discussed in examples 1 and 2.

' Example1 shows how employ the use of a structure containing submenu handle/string
' resource ID pairs in an array, and how to pass individual elements of the array to
' MenuHelp's lpwIDs parameter. The hMainMenu parameter is used in this example.
' This method is easier to code but harder to understand.

' Example2 shows how to explicitly pass the resource string ID to MenuHelp and avoid
' any calculation. The hMainMenu parameter is not used in this example. Is slightly more
' involved to code, but much easier to understand.

